/** Put here CSS that should apply to BOTH VIEW AND EDIT SCREENS. **/

html, body {
    font: 16px/24px "Nobile", "Lucida Grande", Lucida, Verdana, sans-serif;
    margin: 0;
    padding: 0;
}

/* Switch mode box */

#switch-mode-box {
    position: fixed;
    top: 3.5em;
    left: 0;
    padding: 0.2em 0.5em;
    background-color: #333;
    color: #fff;
    font-weight: bold;
    text-transform: uppercase;
}

#switch-mode-box a {
    text-decoration: none;
    color: #fff;
}

/* User box */

#user-box {
    position: absolute;
    top: 0;
    right: 2em;
    padding: 0.2em 0.8em;
    background-color: #333;
    color: #fff;
}

div.username { 
    background: transparent none no-repeat top left;
    padding-left: 30px;
}

#user-box a.dropdown-trigger {
    color: #fff;
}

#user-box .dropdown a, #user-box .dropdown a:visited {
    color: #fff;
}

#user-box .dropdown a:hover {
    color: #bbb;
}

/* Messages */

.message {
    background-color: #bbb;
    padding: 0.5em 1em;
    margin: 1em 0;
    width: 40em;
    text-align: center;
    font-weight: bold;
}

.message.notice {
    background-color: #edc06b;
}

.message.success {
    background-color: #b5cc7a;
}

.message.error {
    background-color: #ff614e;
}

/* Nav */

.top-navigation {
    display: block;
    padding-top: 2em;
    padding-left: 4em;
    background-color: #eee;
}

.tabs1 ul {
	list-style: none;
	padding: 0;
	margin: 0;
	float: left;
	width: 100%;
}

.tabs1 li {
	float: left;
	margin: 0 1em 0 0;
}

.tabs1 a {
	text-decoration: none;
    white-space: nowrap;
	display: block;
	width: 7em;
	padding: 0 0.5em;
	font-weight: bold;
	color: #333;
	border-bottom: 0.5em solid #ccc;
}

.tabs1 a:hover, .tabs1 a.selected {
	color: #000;
	border-color: #555;
}

/* Dropdown styles */

.dropdown {
    display: none;
    z-index: 1;
	opacity: .95;
	filter: alpha(opacity=95);
	-ms-filter:"progid:DXImageTransform.Microsoft.Alpha(Opacity=95)";
}

.dropdown ul {
    margin: 0;
    padding: 0;
    list-style: none;
}

.dropdown a {
    display: block;
    padding: 0.2em 1em;
    text-decoration: none;
}

.dropdown-trigger {
    text-decoration: none;
}

/* Misc generic styles */

.hidden {
    display: none;
}

.clear {
    clear: both;
}

.vertical-text {
    -moz-transform:rotate(-270deg); 
    -moz-transform-origin: bottom left;
    -webkit-transform: rotate(-270deg);
    -webkit-transform-origin: bottom left;
    -o-transform: rotate(-270deg);
    -o-transform-origin:  bottom left;
    filter: progid:DXImageTransform.Microsoft.BasicImage(rotation=1);
}

.round-corners-top {
    -webkit-border-top-left-radius: 10px;
    -webkit-border-top-right-radius: 10px;
    -moz-border-radius-topleft: 10px;
    -moz-border-radius-topright: 10px;
    border-top-left-radius: 10px;
    border-top-right-radius: 10px;
}

.round-corners-bottom {
    -webkit-border-bottom-left-radius: 10px;
    -webkit-border-bottom-right-radius: 10px;
    -moz-border-radius-bottomleft: 10px;
    -moz-border-radius-bottomright: 10px;
    border-bottom-left-radius: 10px;
    border-bottom-right-radius: 10px;
}

.round-corners {
    -webkit-border-bottom-left-radius: 7px;
    -webkit-border-bottom-right-radius: 7px;
    -webkit-border-top-left-radius: 7px;
    -webkit-border-top-right-radius: 7px;
    -moz-border-radius-bottomleft: 7px;
    -moz-border-radius-bottomright: 7px;
    -moz-border-radius-topleft: 7px;
    -moz-border-radius-topright: 7px;
    border-bottom-left-radius: 7px;
    border-bottom-right-radius: 7px;
    border-top-left-radius: 7px;
    border-top-right-radius: 7px;
}
