/** Put here CSS that should apply to EDIT SCREENS ONLY. **/

.top-navigation {
    background-color: #edc06b;
}

#main {
    margin: 2em;
    margin-left: 4em;
}

.hover-link {
    background-color: #ccc;
    cursor: pointer;
}

/* Breadcrumbs */

.breadcrumbs {
    display: block;
    padding-bottom: 1em;
    color: #333;
}

.breadcrumbs .you-are-here {
    margin-right: 0.5em;
}

.breadcrumbs a {
    color: #333;
    text-decoration: none;
}

.breadcrumbs a:visited {
    color: #333;
}

.breadcrumbs .dropdown-trigger {
    background-color:  #eee;
    border: 1px solid #aaa;
    padding: 0.1em 0.5em;
    z-index: 5;
}

.breadcrumbs .dropdown {
    position: absolute;
    border: 1px solid #aaa;
    background-color: #eee;
}

.breadcrumbs .dropdown h4 {
    margin: 0 0.5em 0 0.5em;
    padding: 0.2em;
}

.breadcrumbs .dropdown a:hover {
    background-color: white;
}

/* Form styles */

div.form {
    width: 40em;
    background-color:  #eee;
    border: 1px solid #aaa;
    padding: 1em;
    margin: 1em 1em 1em 0;
}

div.form h2 {
    margin-top: 0;
}

form ul {
    margin: 0;
    padding: 0;
    list-style-stype: none;
}

form li {
	display: block;
    padding: 1em;
}

form li.error {
    padding: 1em;
}

.deformClosebuttonActive {
   border: 0;
   cursor: pointer;
   float: left;
   /* width: 1em; */
   /* height: 1em; */
   /* margin: .5em; */
}

.deformClosebuttonActive:before {
    content:"✖";
    color:red;
    margin-right: 2px;
}

form label.desc {
    display: block;
    margin: 0 0 0.5em 0;
    font-weight: bold;
}

form fieldset {
    margin: 0;
    padding: 0;
    border-width: 0;
}

.inline {
	margin: 0;
	padding: 0;
}

.errorLi {
    display: none;
}

form li.error {
    background-color: #ff614e;
}

/* Table styles */

table {
    text-align: left;
}

table td, table th {
    padding: 0.5em 0.5em;
}

table th {
    color: #555;
}

td.checkbox {
    text-align: center;
}

tr.even {
    background-color: white;
}

/* Login form */

.login-form .forgot-password {
    text-align: center;
}

/* jquery.toastmessage */

.toast-position-top-right {
    top: 3em;
    right: 2em;
}