/** Put here CSS that should apply to VIEW SCREENS ONLY. **/

#main {
    margin: 2em;
    margin-left: 4em;
}

.has-leftcol {
    margin-left: 13em;
}

.has-rightcol {
    margin-right: 17em;
}

.right-column {
    float: right;
    width: 12em;
}

.left-column {
    float: left;
    width: 12em;
}

#footer {
    clear: both;
}

.slot {
    list-style: none;
    padding: 0;
    margin: 0;
}

.slot > li {
    margin-bottom: 2em;
}

#local-navigation ul {
    list-style: none;
    padding: 0;
    margin: 0;
    width: 100%;
}

#local-navigation li {
    padding: 0.5em 0.2em;
}

#local-navigation a {
    display: block;
    white-space: nowrap;
    text-decoration: none;
    font-weight: bold;
    color: #333;
}

#local-navigation a.selected {
    font-style: italic;
}

.content .description {
    font-weight: bold;
}

